#
# gui.py
#
# Python Digital Audio Recorder - GUI module
#
# Copyright (C) 2003 Daniel Thompson <d.thompson@gmx.net> 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#

import os, os.path, pickle, string, sys, time

import pygtk
pygtk.require("2.0")
import gobject, gtk, gtk.glade

class PydarGui:
	""" Class to control the main pydar window """

	def __init__(self, interface, prefs, tuner):
		self.tuner = tuner
		self.prefs = prefs

		# open the widget tree and initialize it with this class
		self.widgetTree = gtk.glade.XML(interface)
		self.widgetTree.signal_autoconnect(self)

		# set the headings for the main list
		self.listModel = gtk.ListStore(
			gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING,
			gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_PYOBJECT
		)
		treeView = self.get_widget("main_treeview1")
		treeView.set_model(self.listModel)
		i = 0
		for heading in "Station", "Date", "Start time", "End time", "Filename":
			renderer = gtk.CellRendererText()
			column = gtk.TreeViewColumn(heading, renderer, text=i)
			column.set_resizable(gtk.TRUE)
			treeView.append_column(column)

			# generate a constant to reference this column
			self.__dict__[heading.replace(' ', '_').upper()] = i

			i += 1
		self.ID = i
			
		# cache some of the most frequently looked up names
		self.programme_spinbuttons = (
			self.get_widget("programme_spinbutton1"),
			self.get_widget("programme_spinbutton2"),
			self.get_widget("programme_spinbutton3"),
			self.get_widget("programme_spinbutton4"),
		)

		# register each of the items in the GUI
		valid_ids = self.tuner.get_id_list()
		for id in self.prefs.get_programme_ids():
			if id in valid_ids:
				self.add_programme(id, self.prefs.get_programme(id))
			else:
				self.prefs.remove_programme(id)

		# create a class instance to manage the preferences dialog
		self.preferences = PydarPreferences(self.widgetTree, prefs, tuner)

	def update_model(self, iter, id, programme):
		lm = self.listModel
		station, (day, month, year), start, end, fname = programme
		lm.set_value(iter, self.STATION,    station)
		lm.set_value(iter, self.DATE,       "%2d/%2d/%4d" % (day, month, year))
		lm.set_value(iter, self.START_TIME, "%2d:%02d" % start)
		lm.set_value(iter, self.END_TIME,   "%2d:%02d" % end)
		lm.set_value(iter, self.FILENAME,   fname)
		lm.set_value(iter, self.ID,         id)
		
	def add_programme(self, id, programme):
		lm = self.listModel
		i = lm.append()
		self.update_model(i, id, programme)

	def get_widget(self, name):
		""" short hand to obtain named widgets """
		return self.widgetTree.get_widget(name)
	
	def beep(self):
		sys.stdout.write('%c' % 7)
		sys.stdout.flush()

	def on_about_open(self, widget):
		""" display and manage the about dialog box """
		d = self.get_widget("about")
		d.run()
		d.hide()
		
	def on_main_quit_activate(self, widget, *extras):
		""" handle user request to close the application """
		if not self.tuner.is_videodev_usable():
			md = gtk.MessageDialog(self.get_widget("main"),
					       gtk.DIALOG_MODAL, # | gtk.DIALOG_NO_SEPARATOR,
					       gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
					       "The radio device is currently in use.\n\n"
					       "Pydar is not able to schedule recordings while "
					       "the radio is in use.")
			md.run()
			md.destroy()

		gtk.mainquit()

	def on_new_programme_open(self, widget):
		# initalize the various widgets
		timespec = time.localtime(time.time())
		(hh, mm, ehh, emm) = self.programme_spinbuttons
		hh.set_value(timespec[3])
		mm.set_value(timespec[4])
		# TODO: this is not 'midnight safe'
		ehh.set_value(timespec[3]+1)
		emm.set_value(timespec[4])

		filename = self.get_widget("programme_entry1")
		filename.set_text("")
		self.on_programme_filename_changed(filename)

		# run the dialog
		programme = self.on_programme_open(widget, "New Programme")
		if None == programme:
			return

		# get the tuner backend to record the program, add it to the GUI and record our state
		id = self.tuner.add_programme(programme)
		self.add_programme(id, programme)
		self.prefs.set_programme(id, programme)

	def do_modify_selection(self, listModel, index, iter):
		self.__modify_flag = False

		id = listModel.get_value(iter, self.ID)
		station, (day, month, year), (h, m), (eh, em), fname = self.prefs.get_programme(id)

		calendar = self.get_widget("programme_calendar1")
		calendar.select_month(month-1, year)
		calendar.select_day(day)

		(hh, mm, ehh, emm) = self.programme_spinbuttons
		hh.set_value(h)
		mm.set_value(m)
		ehh.set_value(eh)
		emm.set_value(em)

		filename = self.get_widget("programme_entry1")
		filename.set_text(fname)
		self.on_programme_filename_changed(filename)
	
		# run the dialog
		programme = self.on_programme_open(listModel, "Modify Programme", station)
		if None == programme:
			return

		# delete the existing entry
		self.tuner.delete_programme(id)
		self.prefs.remove_programme(id)

		# recreate a new entry
		id = self.tuner.add_programme(programme)
		self.update_model(iter, id, programme)
		self.prefs.set_programme(id, programme)

	def on_modify_programme_open(self, widget):
		tv = self.get_widget("main_treeview1")
		selection = tv.get_selection()
		self.__modify_flag = True
		selection.selected_foreach(self.do_modify_selection)
		if self.__modify_flag:
			self.beep()

	def on_programme_open(self, widget, title, select=None):
		""" display and manage the programme dialog box """
		d = self.get_widget("programme")
		d.set_title(title)
		om = self.get_widget("programme_optionmenu1")
		filename = self.get_widget("programme_entry1")

		# construct the station menu
		menu = gtk.Menu()
		stations = self.prefs.get_station_names()
		i = 0
		for s in stations:
			mi = gtk.MenuItem(s)
			mi.show()
			menu.append(mi)
			if select == s:
				menu.set_active(i)
			i += 1
		om.set_menu(menu)

		# preform the dialog
		response = d.run()
		d.hide()

		# ignore the results if we were not dismissed by an OK
		if response != gtk.RESPONSE_OK:
			return

		# extract the values from the dialog
		station = om.get_children()[0].get()
		(year, month, day) = self.get_widget("programme_calendar1").get_date()
		month += 1		# in GTK month is an enumeration and starts at 0!
		start = (self.programme_spinbuttons[0].get_value_as_int(),
		         self.programme_spinbuttons[1].get_value_as_int())
		end   = (self.programme_spinbuttons[2].get_value_as_int(),
		         self.programme_spinbuttons[3].get_value_as_int())
		# get filename and make it absolute
		fname =  os.path.abspath(filename.get_text())

		# pack as a tuple and return the recording
		return (station, (day, month, year), start, end, fname)

	def on_programme_filename_changed(self, widget):
		""" validate the filename and grey the OK button if it is wrong """
		ok = self.get_widget("programme_okbutton1")
		if widget.get_text() == "":
			ok.set_sensitive(gtk.FALSE)
		else:
			ok.set_sensitive(gtk.TRUE)

	def on_programme_fileselector_open(self, widget):
		""" display a fileselector and apply the results to the programme dialog """
		entry = self.get_widget("programme_entry1")
		self.do_fileselector_for_entry(entry)

	def do_delete_selection(self, listModel, index, iter):
		if self.__delete_flag:
			md = gtk.MessageDialog(self.get_widget("main"),
					       gtk.DIALOG_MODAL, # | gtk.DIALOG_NO_SEPARATOR,
					       gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO,
					       "Remove all currently selected programmes?")
			response = md.run()
			md.destroy()
			if response != gtk.RESPONSE_YES:
				self.__delete_flag = None
			else:
				self.__delete_flag = False
		
		if None == self.__delete_flag:
			return

		# get the id of the programme to delete
		id = listModel.get_value(iter, self.ID)

		# we must remove if from the tuner first (if it fails it will generate an
		# exception)
		self.tuner.delete_programme(id)

		# now remove it from the GUI and the stored state
		listModel.remove(iter)
		self.prefs.remove_programme(id)

	def on_delete_programme(self, widget):
		""" remove recorded programmes from the list (and the tuner) """
		tv = self.get_widget("main_treeview1")
		selection = tv.get_selection()
		self.__delete_flag = True
		selection.selected_foreach(self.do_delete_selection)
		if self.__delete_flag:
			self.beep()
	
	def do_fileselector_for_entry(self, entry):
		fs = gtk.FileSelection()
		fs.set_filename(entry.get_text())
		response = fs.run()
		if response == gtk.RESPONSE_OK:
			entry.set_text(fs.get_filename())
		fs.destroy()

class PydarPreferences:
	""" Class to manage the Pydar prefences dialog """

	def __init__(self, widgetTree, prefs, tuner):
		self.widgetTree = widgetTree
		self.widgetTree.signal_autoconnect(self)

		self.prefs = prefs
		self.tuner = tuner

		# initialize the presets and store some useful links
		i = 1
		for name, freq in prefs.get_stations():
			entry = self.get_widget('presets_entry%d' % i)
			entry.set_data('index', i)
			entry.set_text(name)
			spinbutton = self.get_widget('presets_spinbutton%d' % i)
			spinbutton.set_data('index', i)
			spinbutton.set_value(freq)
			i += 1

		# initialize the sliders
		for i in ('vorbis_quality', 'speex_quality'):
			widget = self.get_widget('preferences_' + i)
			value = self.prefs.get_preference(i)
			widget.set_value(value)
			widget.set_data('pref', i)

		# initialize the speex radiobuttons
		rate = prefs.get_preference('speex_rate')
		for i in ('8000', '16000', '32000'):
			widget = self.get_widget('preferences_speex_rate_' + i)
			if rate == i:
				widget.set_active(gtk.TRUE)
			widget.set_data('pref', 'speex_rate')
			widget.set_data('value', i)
		
		# initialize the rest of the preferences
		self.get_widget('preferences_radiodevice').set_text(prefs.get_preference('radio'))

	# TODO: this should be factored out into a common super-class
	def get_widget(self, name):
		""" short hand to obtain named widgets """
		return self.widgetTree.get_widget(name)

	def on_preferences1_activate(self, widget):
		d = self.get_widget('preferences')
		d.show()

	def on_preferences_close(self, widget, event=None):
		d = self.get_widget('preferences')
		d.hide()
		
	def on_preferences_radiobutton_clicked(self, widget, event=None):
		pref = widget.get_data('pref')
		self.prefs.set_preference(pref, widget.get_data('value'))

	def on_preferences_radiodevice_commit(self, widget, event=None):
		self.prefs.set_preference('radio', widget.get_text())

	def on_preferences_slider_commit(self, widget, event=None):
		pref = widget.get_data('pref')
		self.prefs.set_preference(pref, widget.get_value())

	def on_presets_entry_commit(self, widget, event=None):
		i = widget.get_data('index')
		name = widget.get_text()
		freq = self.get_widget('presets_spinbutton%d' % i).get_value()
		self.prefs.set_station(i-1, name, freq)

	def on_presets_spinbutton_commit(self, widget, event=None):
		i = widget.get_data('index')
		name = self.get_widget('presets_entry%d' % i).get_text()
		freq = widget.get_value()
		self.prefs.set_station(i-1, name, freq)
