#
# prefs.py
#
# Python Digital Audio Recorder - Manage persistant state.
#
# Copyright (C) 2003 Daniel Thompson <d.thompson@gmx.net> 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#

import pickle

class Preferences:
	""" Class to look after the persistant state.

	This class has a deliberately baroque interface to ensure we always have sufficient
	information to save our state whenever it is modified.

	This class does not contain any GUI code and can be used from processes
	spawned by the at daemon.
	"""

	# this are returned if there is a KeyError looking up in the normal
	# preferences dictionary. having a secondary table makes transitioning
	# between different versions less effort.
	__default_preferences = {
		'radio': '/dev/radio',
		'vorbis_quality': 3,
		'speex_quality': 3,
		'speex_rate' : '16000',
	}

	def __init__(self, state):
		self.__stateFile = state

		try:
			self.__load_state()
		except IOError:
			# state file does not exist - THIS IS NOT AN ERROR
			self.__new_state()
		except EOFError:
			print "Pydar's stateFile is corrupt, starting from scratch"
			self.__new_state()

	def __load_state(self):
		f = open(self.__stateFile, 'r')
		self.__programmes = pickle.load(f)
		self.__preferences = pickle.load(f)
		self.__stations = pickle.load(f)
		f.close()

		# custom sort the presets into (almost) alphabetic order
		def preset_sort(a, b):
			if a[0] == '': return 1
			if b[0] == '': return -1
			return cmp(a, b)
		self.__stations.sort(preset_sort)
	
	def __save_state(self):
		# TODO: this is not safe if the disk is full!
		f = open(self.__stateFile, 'w')
		pickle.dump(self.__programmes, f)
		pickle.dump(self.__preferences, f)
		pickle.dump(self.__stations, f)
		f.close()


	def __new_state(self):
		self.__programmes = {}

		self.__preferences = {}

		self.__stations = []
		# speculatively try to extract the gnomeradio presets
		try:
			self.__query_gnomeradio()
		except:
			raise
			pass

		for i in range(len(self.__stations), 10):
			self.__stations.append(('', 88.0 + i*2))

	def __query_gnomeradio(self):
		""" use gconf to extract gnomeradio's station presets """
		import gconf

		client = gconf.client_get_default()
		key = '/apps/gnomeradio/presets'

		# register which directory we are interested in
		client.add_dir(key, gconf.CLIENT_PRELOAD_NONE)

		presets = client.get_int(key + '/presets')
		if 0 == presets:
			raise NameError, 'gconf key does not exist'

		for i in range(presets):
			name = client.get_string(key + '/%d/name' % i)
			freq = client.get_float(key + '/%d/freqency' % i)
			self.__stations.append((name, freq))

	def get_stations(self):
		""" obtain an (immutable) sequence of all preset radio stations """
		return tuple(self.__stations)

	def get_station_names(self):
		""" obtain a list of just the station names """
		return [ t[0] for t in self.__stations if t[0] != '' ]

	def get_station_by_name(self, name):
		station = [ t for t in self.__stations if t[0] == name ]
		return station[0][1]

	def set_station(self, id, name, freq):
		""" set a radio station preset corresponding to its position in the station list """
		self.__stations[id] = (name, freq)
		self.__save_state()

	def get_programme_ids(self):
		""" obtain a list of the (at daemon) ids for each programme in the record queue """
		return self.__programmes.keys()
	
	def get_programme(self, id):
		""" obtain the details of a specific programme """
		return self.__programmes[id]

	def set_programme(self, id, programme):
		""" create/modify the details of a specific programme """
		self.__programmes[id] = programme
		self.__save_state()

	def remove_programme(self, id):
		del self.__programmes[id]
		self.__save_state()
	
	def get_preference(self, id):
		try:
			return self.__preferences[id]
		except KeyError:
			return self.__default_preferences[id]

	def set_preference(self, id, value):
		self.__preferences[id] = value
		self.__save_state()
	
	def remove_preference(self, id):
		del self.__preferences[id]
		self.__save_state()

if __name__ == '__main__':
	fname = 'test.state'

	p = Preferences(fname)
	p.set_preference('oggenc_quality', 5.0)
	p.set_station(9, 'TestStation', 108.0)
	print p.get_station_by_name('TestStation')

	f = open(fname)
	print pickle.load(f)
	print pickle.load(f)
	print pickle.load(f)
	f.close()

